@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Recursion into other modules.
 
@@ -38,16 +27,15 @@ from logging import info, warning
 
 _warned_about = set()
 
-imported_modules = {}
-
 def recurseTo( module_package, module_filename, module_relpath ):
-    if module_relpath not in imported_modules:
+    if TreeBuilding.isImportedPath( module_relpath ):
         info( "Recurse to import %s", module_relpath )
 
         try:
-            imported_module = TreeBuilding.buildModuleTree(
+            _imported_module = TreeBuilding.buildModuleTree(
                 filename = module_filename,
                 package  = module_package,
+                is_top   = False,
                 is_main  = False
             )
         except ( SyntaxError, IndentationError ) as e:
@@ -65,11 +53,9 @@ def recurseTo( module_package, module_filename, module_relpath ):
 
         assert not module_relpath.endswith( "/__init__.py" )
 
-        imported_modules[ module_relpath ] = imported_module
-
-        return imported_modules[ module_relpath ], True
+        return TreeBuilding.getImportedModule( module_relpath ), True
     else:
-        return imported_modules[ module_relpath ], False
+        return TreeBuilding.getImportedModule( module_relpath ), False
 
 
 def considerFilename( module_filename, module_package ):
@@ -103,7 +89,12 @@ def checkPluginPath( plugin_filename, module_package ):
 
         if module:
             if not added:
-                warning( "Recursed to module at '%s' twice.", plugin_info[0] )
+                warning(
+                    "Recursed to %s '%s' at '%s' twice.",
+                    "package" if module.isPackage() else "module",
+                    module.getName(),
+                    plugin_info[0]
+                )
 
             if module.isPackage():
                 package_dir = Utils.dirname( module.getFilename() )
@@ -112,6 +103,8 @@ def checkPluginPath( plugin_filename, module_package ):
                     if sub_filename == "__init__.py":
                         continue
 
+                    assert sub_path != plugin_filename, package_dir
+
                     if Importing.isPackageDir( sub_path ) or sub_path.endswith( ".py" ):
                         checkPluginPath( sub_path, module.getFullName() )
 
