@@ -31,6 +31,7 @@ from .templates.CodeTemplatesParameterParsing import *
 from .templates.CodeTemplatesExceptions import *
 from .templates.CodeTemplatesImporting import *
 from .templates.CodeTemplatesPrinting import *
+from .templates.CodeTemplatesBranches import *
 from .templates.CodeTemplatesTuples import *
 from .templates.CodeTemplatesLists import *
 from .templates.CodeTemplatesDicts import *
@@ -39,83 +40,6 @@ from .templates.CodeTemplatesLoops import *
 
 from .templates.CodeTemplatesExecEval import *
 
-# We have some very long lines in here that should not be shorter though.
-# pylint: disable=C0301
-
-try_finally_template = """\
-_PythonExceptionKeeper _caught_%(try_count)d;
-bool _continue_%(try_count)d = false;
-bool _break_%(try_count)d = false;
-bool _return_%(try_count)d = false;
-
-try
-{
-%(tried_code)s
-}
-catch ( _PythonException &_exception )
-{
-    if ( !_exception.hasTraceback() )
-    {
-        _exception.setTraceback( %(tb_making)s );
-    }
-    else if ( traceback == false )
-    {
-        _exception.addTraceback( frame_guard.getFrame() );
-    }
-    traceback = true;
-
-    _caught_%(try_count)d.save( _exception );
-
-    frame_guard.detachFrame();
-}
-catch ( ContinueException &e )
-{
-    _continue_%(try_count)d = true;
-}
-catch ( BreakException &e )
-{
-    _break_%(try_count)d = true;
-}
-catch ( ReturnException &e )
-{
-    _return_%(try_count)d = true;
-}
-
-// Final code:
-%(final_code)s
-
-_caught_%(try_count)d.rethrow();
-
-if ( _continue_%(try_count)d )
-{
-    throw ContinueException();
-}
-if ( _break_%(try_count)d )
-{
-    throw BreakException();
-}
-if ( _return_%(try_count)d )
-{
-    throw ReturnException();
-}"""
-
-
-template_branch_one = """\
-if ( %(condition)s )
-{
-%(branch_code)s
-}"""
-
-template_branch_two = """\
-if ( %(condition)s )
-{
-%(branch_yes_code)s
-}
-else
-{
-%(branch_no_code)s
-}"""
-
 def enableDebug():
     templates = dict( globals() )
 
