@@ -77,7 +77,7 @@ def namifyConstant( constant ):
     elif type( constant ) is float:
         return "float_%s" % repr( constant ).replace( ".", "_" ).replace( "-", "_minus_" ).replace( "+", "" )
     elif type( constant ) is complex:
-        value = str( constant ).replace( "+", "p" ).replace( "-", "m" ).replace(".","_")
+        value = str( constant ).replace( "+", "p" ).replace( "-", "m" ).replace( ".", "_" )
 
         if value.startswith( "(" ) and value.endswith( ")" ):
             value = value[1:-1]
@@ -110,7 +110,7 @@ def namifyConstant( constant ):
                 for value in constant:
                     parts.append( namifyConstant( value ) )
 
-                return result + "_".join( parts )
+                return result + "_".join( parts ) + "_tuple"
             except ExceptionCannotNamify:
                 warning( "Couldn't namify '%r'" % value )
 
@@ -127,7 +127,7 @@ def namifyConstant( constant ):
                 for value in constant:
                     parts.append( namifyConstant( value ) )
 
-                return result + "_".join( parts )
+                return result + "_".join( parts )  + "_list"
             except ExceptionCannotNamify:
                 warning( "Couldn't namify '%r'" % value )
 
