@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Operator code tables
 
@@ -33,14 +22,22 @@ These are mostly used to look up the Python C/API from operations or a wrapper u
 """
 
 binary_operator_codes = {
-    "Add"       : "PyNumber_Add",
-    "Sub"       : "PyNumber_Subtract",
-    "Pow"       : "PyNumber_Power",
-    "Mult"      : "PyNumber_Multiply",
-    "Div"       : "PyNumber_Divide",
+# Those commented out in this section have fully specialized variants already.
+
+#    "Add"       : "PyNumber_Add",
+#    "Sub"       : "PyNumber_Subtract",
+#    "Div"       : "PyNumber_Divide",
+#    "Mult"      : "PyNumber_Multiply",
+#    "Mod"       : "PyNumber_Remainder",
+# These have their own variants only to make sure the generic code is inlined
+# but the CPython code is not inlined.
+
+#    "Pow"       : "PyNumber_Power",
+#    "IPow"      : "PyNumber_InPlacePower",
+
+# The others are generic code and would be faster if they had a specialized variant too.
     "FloorDiv"  : "PyNumber_FloorDivide",
     "TrueDiv"   : "PyNumber_TrueDivide",
-    "Mod"       : "PyNumber_Remainder",
     "LShift"    : "PyNumber_Lshift",
     "RShift"    : "PyNumber_Rshift",
     "BitAnd"    : "PyNumber_And",
@@ -48,7 +45,6 @@ binary_operator_codes = {
     "BitXor"    : "PyNumber_Xor",
     "IAdd"      : "PyNumber_InPlaceAdd",
     "ISub"      : "PyNumber_InPlaceSubtract",
-    "IPow"      : "PyNumber_InPlacePower",
     "IMult"     : "PyNumber_InPlaceMultiply",
     "IDiv"      : "PyNumber_InPlaceDivide",
     "IFloorDiv" : "PyNumber_InPlaceFloorDivide",
