@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Tree nodes for builtin references.
 
@@ -62,7 +51,7 @@ class CPythonExpressionBuiltinRefBase( CPythonNodeBase, CPythonExpressionMixin )
     def getBuiltinName( self ):
         return self.builtin_name
 
-    def mayHaveSideEffects( self ):
+    def mayHaveSideEffects( self, constraint_collection ):
         # Referencing the builtin name has no side effect
         return False
 
@@ -90,7 +79,7 @@ class CPythonExpressionBuiltinRef( CPythonExpressionBuiltinRefBase ):
     def getCompileTimeConstant( self ):
         return __builtins__[ self.builtin_name ]
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         quick_names = {
             "None"      : None,
             "True"      : True,
@@ -134,10 +123,16 @@ class CPythonExpressionBuiltinAnonymousRef( CPythonExpressionBuiltinRefBase ):
         # Virtual method, pylint: disable=R0201
         return True
 
+    def mayProvideReference( self ):
+        # No reference provided from this, there are just a global identifiers, or
+        # accesses to them.
+
+        return False
+
     def getCompileTimeConstant( self ):
         return builtin_anon_names[ self.builtin_name ]
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         return self, None, None
 
 
@@ -166,8 +161,13 @@ class CPythonExpressionBuiltinExceptionRef( CPythonExpressionBuiltinRefBase ):
         # Virtual method, pylint: disable=R0201
         return True
 
+    def mayProvideReference( self ):
+        # No reference provided from this, it's just a global identifier.
+
+        return False
+
     def getCompileTimeConstant( self ):
         return builtin_exception_values[ self.builtin_name ]
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         return self, None, None
