@@ -3,39 +3,29 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Nodes that build containers.
 
 """
 
 
-from .NodeBases import CPythonExpressionChildrenHavingBase
+from .NodeBases import CPythonExpressionChildrenHavingBase, CPythonSideEffectsFromChildrenMixin
 
 from .NodeMakingHelpers import getComputationResult, makeConstantReplacementNode
 
-class CPythonExpressionMakeSequenceBase( CPythonExpressionChildrenHavingBase ):
+class CPythonExpressionMakeSequenceBase( CPythonSideEffectsFromChildrenMixin, \
+                                         CPythonExpressionChildrenHavingBase ):
     named_children = ( "elements", )
 
     def __init__( self, sequence_kind, elements, source_ref ):
@@ -67,7 +57,7 @@ class CPythonExpressionMakeSequenceBase( CPythonExpressionChildrenHavingBase ):
         # Abstract method, pylint: disable=R0201,W0613
         return None
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         for element in self.getElements():
             if not element.isExpressionConstantRef() or element.isMutable():
                 return self, None, None
@@ -87,13 +77,19 @@ class CPythonExpressionMakeSequenceBase( CPythonExpressionChildrenHavingBase ):
             )
 
     def isKnownToBeIterable( self, count ):
-        return count == len( self.getElements() )
+        return count is None or count == len( self.getElements() )
 
-    def getUnpacked( self, count ):
-        # For every child except dictionaries, it's this easy.
-        assert count == len( self.getElements() )
+    def getIterationValue( self, count, constraint_collection ):
+        return self.getElements()[ count ]
 
-        return self.getElements()
+    def getIterationLength( self, constraint_collection ):
+        return len( self.getElements() )
+
+    def canPredictIterationValues( self, constraint_collection ):
+        return True
+
+    def getTruthValue( self, constraint_collection ):
+        return self.getIterationLength( constraint_collection ) > 0
 
 
 class CPythonExpressionMakeTuple( CPythonExpressionMakeSequenceBase ):
@@ -159,11 +155,12 @@ class CPythonExpressionKeyValuePair( CPythonExpressionChildrenHavingBase ):
     getKey = CPythonExpressionChildrenHavingBase.childGetter( "key" )
     getValue = CPythonExpressionChildrenHavingBase.childGetter( "value" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         return self, None, None
 
 
-class CPythonExpressionMakeDict( CPythonExpressionChildrenHavingBase ):
+class CPythonExpressionMakeDict( CPythonSideEffectsFromChildrenMixin, \
+                                 CPythonExpressionChildrenHavingBase ):
     kind = "EXPRESSION_MAKE_DICT"
 
     named_children = ( "pairs", )
@@ -179,7 +176,7 @@ class CPythonExpressionMakeDict( CPythonExpressionChildrenHavingBase ):
 
     getPairs = CPythonExpressionChildrenHavingBase.childGetter( "pairs" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         pairs = self.getPairs()
 
         for pair in pairs:
@@ -212,5 +209,17 @@ class CPythonExpressionMakeDict( CPythonExpressionChildrenHavingBase ):
 
         return new_node, "new_constant", "Created dictionary found to be constant."
 
-    def getUnpacked( self, count ):
-        assert False
+    def isKnownToBeIterable( self, count ):
+        return count is None or count == len( self.getPairs() )
+
+    def getIterationLength( self, constraint_collection ):
+        return len( self.getPairs() )
+
+    def canPredictIterationValues( self, constraint_collection ):
+        return True
+
+    def getIterationValue( self, count, constraint_collection ):
+        return self.getPairs()[ count ].getKey()
+
+    def getTruthValue( self, constraint_collection ):
+        return self.getIterationLength( constraint_collection ) > 0
