@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Nodes concern with exec and eval builtins.
 
@@ -43,7 +32,7 @@ from .NodeBases import (
 
 from .NodeMakingHelpers import convertNoneConstantToNone
 
-from nuitka import Variables
+from nuitka import Variables, Utils
 
 class CPythonExpressionBuiltinEval( CPythonExpressionChildrenHavingBase ):
     kind = "EXPRESSION_BUILTIN_EVAL"
@@ -65,24 +54,45 @@ class CPythonExpressionBuiltinEval( CPythonExpressionChildrenHavingBase ):
     getGlobals = CPythonExpressionChildrenHavingBase.childGetter( "globals" )
     getLocals = CPythonExpressionChildrenHavingBase.childGetter( "locals" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         # TODO: Attempt for constant values to do it.
         return self, None, None
 
 
 # Note: Python3 only so far.
-class CPythonExpressionBuiltinExec( CPythonExpressionBuiltinEval ):
-    kind = "EXPRESSION_BUILTIN_EXEC"
+if Utils.python_version >= 300:
+    class CPythonExpressionBuiltinExec( CPythonExpressionBuiltinEval ):
+        kind = "EXPRESSION_BUILTIN_EXEC"
+
+        def needsLocalsDict( self ):
+            return True
+
+# Note: Python2 only
+if Utils.python_version < 300:
+    class CPythonExpressionBuiltinExecfile( CPythonExpressionBuiltinEval ):
+        kind = "EXPRESSION_BUILTIN_EXECFILE"
+
+        named_children = ( "source", "globals", "locals" )
+
+        def __init__( self, source_code, globals_arg, locals_arg, source_ref ):
+            CPythonExpressionBuiltinEval.__init__( self, source_code, globals_arg, locals_arg, source_ref )
+
+        def needsLocalsDict( self ):
+            return True
 
 
-class CPythonExpressionBuiltinExecfile( CPythonExpressionBuiltinEval ):
-    kind = "EXPRESSION_BUILTIN_EXECFILE"
-
-    named_children = ( "source", "globals", "locals" )
-
-    def __init__( self, source_code, globals_arg, locals_arg, source_ref ):
-        CPythonExpressionBuiltinEval.__init__( self, source_code, globals_arg, locals_arg, source_ref )
-
+# TODO: Find a place for this. Potentially as an attribute of nodes themselves.
+def _couldBeNone( node ):
+    if node is None:
+        return True
+    elif node.isExpressionMakeDict():
+        return False
+    elif node.isExpressionBuiltinGlobals() or node.isExpressionBuiltinLocals() or \
+           node.isExpressionBuiltinDir0() or node.isExpressionBuiltinVars():
+        return False
+    else:
+        # assert False, node
+        return True
 
 class CPythonStatementExec( CPythonChildrenHaving, CPythonNodeBase ):
     kind = "STATEMENT_EXEC"
@@ -111,6 +121,11 @@ class CPythonStatementExec( CPythonChildrenHaving, CPythonNodeBase ):
     getGlobals = CPythonChildrenHaving.childGetter( "globals" )
     getLocals = CPythonChildrenHaving.childGetter( "locals" )
 
+    def needsLocalsDict( self ):
+        return _couldBeNone( self.getGlobals() ) or \
+               self.getGlobals().isExpressionBuiltinLocals() or \
+               self.getLocals() is not None and self.getLocals().isExpressionBuiltinLocals()
+
 
 # TODO: This is totally bitrot
 class CPythonStatementExecInline( CPythonChildrenHaving, CPythonClosureTaker, CPythonClosureGiverNodeBase ):
@@ -176,3 +191,6 @@ class CPythonStatementExecInline( CPythonChildrenHaving, CPythonClosureTaker, CP
                 owner         = self.provider,
                 variable_name = variable_name
             )
+
+    def needsLocalsDict( self ):
+        return True
