@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Nodes for unary and binary operations.
 
@@ -38,15 +27,13 @@ from .NodeMakingHelpers import getComputationResult
 
 from nuitka import PythonOperators
 
-class CPythonExpressionOperationBase( CPythonExpressionChildrenHavingBase ):
-    named_children = ( "operands", )
+import math
 
-    def __init__( self, operator, simulator, operands, source_ref ):
+class CPythonExpressionOperationBase( CPythonExpressionChildrenHavingBase ):
+    def __init__( self, operator, simulator, values, source_ref ):
         CPythonExpressionChildrenHavingBase.__init__(
             self,
-            values     = {
-                "operands" : operands
-            },
+            values     = values,
             source_ref = source_ref
         )
 
@@ -66,8 +53,6 @@ class CPythonExpressionOperationBase( CPythonExpressionChildrenHavingBase ):
     def getSimulator( self ):
         return self.simulator
 
-    getOperands = CPythonExpressionChildrenHavingBase.childGetter( "operands" )
-
     def isKnownToBeIterable( self, count ):
         # TODO: Could be true, if the arguments said so
         return None
@@ -76,6 +61,8 @@ class CPythonExpressionOperationBase( CPythonExpressionChildrenHavingBase ):
 class CPythonExpressionOperationBinary( CPythonExpressionOperationBase ):
     kind = "EXPRESSION_OPERATION_BINARY"
 
+    named_children = ( "left", "right" )
+
     def __init__( self, operator, left, right, source_ref ):
         assert left.isExpression() and right.isExpression, ( left, right )
 
@@ -83,11 +70,14 @@ class CPythonExpressionOperationBinary( CPythonExpressionOperationBase ):
             self,
             operator   = operator,
             simulator  = PythonOperators.binary_operator_functions[ operator ],
-            operands   = ( left, right ),
+            values     = {
+                "left"  : left,
+                "right" : right
+            },
             source_ref = source_ref
         )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         operator = self.getOperator()
         operands = self.getOperands()
 
@@ -97,6 +87,23 @@ class CPythonExpressionOperationBinary( CPythonExpressionOperationBase ):
             left_value = left.getCompileTimeConstant()
             right_value = right.getCompileTimeConstant()
 
+            if operator == "Mult" and right.isNumberConstant():
+                iter_length = left.getIterationLength( constraint_collection )
+
+                if iter_length is not None:
+                    if iter_length * right_value > 256:
+                        return self, None, None
+
+                if left.isNumberConstant():
+                    if left.isIndexConstant() and right.isIndexConstant():
+                        # Estimate with logarithm, if the result of number calculations is
+                        # computable with acceptable effort, otherwise, we will have to do
+                        # it at runtime.
+
+                        if left_value != 0 and right_value != 0:
+                            if math.log10( abs( left_value ) ) + math.log10( abs( right_value ) ) > 20:
+                                return self, None, None
+
             return getComputationResult(
                 node        = self,
                 computation = lambda : self.getSimulator()(
@@ -108,10 +115,18 @@ class CPythonExpressionOperationBinary( CPythonExpressionOperationBase ):
         else:
             return self, None, None
 
+    def getOperands( self ):
+        return ( self.getLeft(), self.getRight() )
+
+    getLeft = CPythonExpressionChildrenHavingBase.childGetter( "left" )
+    getRight = CPythonExpressionChildrenHavingBase.childGetter( "right" )
+
 
 class CPythonExpressionOperationUnary( CPythonExpressionOperationBase ):
     kind = "EXPRESSION_OPERATION_UNARY"
 
+    named_children = ( "operand", )
+
     def __init__( self, operator, operand, source_ref ):
         assert operand.isExpression(), operand
 
@@ -119,11 +134,13 @@ class CPythonExpressionOperationUnary( CPythonExpressionOperationBase ):
             self,
             operator   = operator,
             simulator  = PythonOperators.unary_operator_functions[ operator ],
-            operands   = ( operand, ),
+            values     = {
+                "operand" : operand
+            },
             source_ref = source_ref
         )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         operator = self.getOperator()
         operand = self.getOperand()
 
@@ -139,11 +156,10 @@ class CPythonExpressionOperationUnary( CPythonExpressionOperationBase ):
         else:
             return self, None, None
 
-    def getOperand( self ):
-        operands = self.getOperands()
+    getOperand = CPythonExpressionChildrenHavingBase.childGetter( "operand" )
 
-        assert len( operands ) == 1
-        return operands[ 0 ]
+    def getOperands( self ):
+        return ( self.getOperand(), )
 
 
 class CPythonExpressionOperationNOT( CPythonExpressionOperationUnary ):
@@ -157,6 +173,39 @@ class CPythonExpressionOperationNOT( CPythonExpressionOperationUnary ):
             source_ref = source_ref
         )
 
+    def getTruthValue( self, constraint_collection ):
+        result = self.getOperand().getTruthValue( constraint_collection )
+
+        return None if result is None else not result
+
+    def mayHaveSideEffects( self, constraint_collection ):
+        operand = self.getOperand()
+
+        if operand.mayHaveSideEffects( constraint_collection ):
+            return True
+
+        # TODO: Find the common ground of these, and make it an expression method.
+        if operand.isExpressionMakeSequence():
+            return False
+
+        if operand.isExpressionMakeDict():
+            return False
+
+        return True
+
+    def extractSideEffects( self ):
+        operand = self.getOperand()
+
+        # TODO: Find the common ground of these, and make it an expression method.
+        if operand.isExpressionMakeSequence():
+            return self.getOperand().extractSideEffects()
+
+        if operand.isExpressionMakeDict():
+            return self.getOperand().extractSideEffects()
+
+        return ( self, )
+
+
 class CPythonExpressionOperationBinaryInplace( CPythonExpressionOperationBinary ):
     kind = "EXPRESSION_OPERATION_BINARY_INPLACE"
 
@@ -171,6 +220,6 @@ class CPythonExpressionOperationBinaryInplace( CPythonExpressionOperationBinary
             source_ref = source_ref
         )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         # TODO: Inplace operation requires extra care to avoid corruption of values.
         return self, None, None
